@@ -1,5 +1,3 @@
-import os
-
 from PyQt4 import QtGui
 from PyQt4 import QtCore
 from PyQt4.QtCore import SIGNAL
@@ -14,7 +12,6 @@ from cola import signals
 from cola.controllers.selectcommits import select_commits
 from cola.models import gitrepo
 from cola.views import repo
-from cola.views import standard
 from cola.compat import set
 
 
@@ -61,9 +58,10 @@ class ClassicController(QtCore.QObject):
             return
         self.updated.add(path)
         gitrepo.GitRepoEntryManager.entry(path).update()
+        entry = gitrepo.GitRepoEntryManager.entry
         for row in xrange(item.rowCount()):
             path = item.child(row, 0).path
-            gitrepo.GitRepoEntryManager.entry(path).update()
+            entry(path).update()
 
     def difftool_predecessor(self, paths):
         """Prompt for an older commit and launch difftool against it."""
@@ -79,7 +77,6 @@ class ClassicController(QtCore.QObject):
 
 if __name__ == '__main__':
     import sys
-    from PyQt4 import QtGui
 
     app = QtGui.QApplication(sys.argv)
     cola_classic()
