@@ -5,7 +5,7 @@ import cola
 import cola.app
 from cola import i18n
 from cola import core
-from cola import gitcmd
+from cola import git
 from cola import gitcmds
 from cola import qtutils
 from cola import signals
@@ -134,7 +134,7 @@ def cherry_pick():
     cola.notifier().broadcast(signals.cherry_pick, commits)
 
 
-def clone_repo(parent, spawn=True):
+def clone_repo(spawn=True):
     """
     Present GUI controls for cloning a repository
 
@@ -142,7 +142,7 @@ def clone_repo(parent, spawn=True):
 
     """
     url, ok = qtutils.prompt('Path or URL to clone (Env. $VARS okay)')
-    url = os.path.expandvars(url)
+    url = os.path.expandvars(core.encode(url))
     if not ok or not url:
         return None
     try:
@@ -168,12 +168,14 @@ def clone_repo(parent, spawn=True):
         return None
 
     # Prompt the user for a directory to use as the parent directory
+    parent = QtGui.QApplication.instance().activeWindow()
     msg = 'Select a parent directory for the new clone'
     dirname = qtutils.opendir_dialog(parent, msg, cola.model().getcwd())
     if not dirname:
         return None
     count = 1
-    destdir = os.path.join(dirname, default)
+    dirname = core.decode(dirname)
+    destdir = os.path.join(dirname, core.decode(default))
     olddestdir = destdir
     if os.path.exists(destdir):
         # An existing path can be specified
@@ -186,7 +188,7 @@ def clone_repo(parent, spawn=True):
     while os.path.exists(destdir):
         destdir = olddestdir + str(count)
         count += 1
-    cola.notifier().broadcast(signals.clone, url, destdir,
+    cola.notifier().broadcast(signals.clone, core.decode(url), destdir,
                               spawn=spawn)
     return destdir
 
@@ -212,9 +214,9 @@ def diff_branch():
                                cola.model().tags)
     if not branch:
         return
-    zfiles_str = gitcmd.instance().diff(branch, name_only=True,
-                                        no_color=True,
-                                        z=True).rstrip('\0')
+    zfiles_str = git.instance().diff(branch, name_only=True,
+                                     no_color=True,
+                                     z=True).rstrip('\0')
     files = zfiles_str.split('\0')
     filename = choose_from_list('Select File', files)
     if not filename:
@@ -241,7 +243,7 @@ def goto_grep(line):
 
 def grep():
     """Prompt and use 'git grep' to find the content."""
-    # This should be a command in cola.commands.
+    # This should be a command in cola.cmds.
     txt, ok = qtutils.prompt('grep')
     if not ok:
         return
@@ -280,9 +282,9 @@ def rebase():
     if not branch:
         return
     #TODO cmd
-    status, output = gitcmd.instance().rebase(branch,
-                                              with_stderr=True,
-                                              with_status=True)
+    status, output = git.instance().rebase(branch,
+                                           with_stderr=True,
+                                           with_status=True)
     qtutils.log(status, output)
 
 
