@@ -1,10 +1,9 @@
 from cola import git
-from cola import gitcmds
 from cola import utils
 
 # put subject at the end b/c it can contain
 # any number of funky characters
-logfmt = 'format:%H%x00%P%x00%d%x00%an%x00%aD%x00%s'
+logfmt = 'format:%H%x01%P%x01%d%x01%an%x01%aD%x01%s'
 git = git.instance()
 
 
@@ -51,7 +50,7 @@ class Commit(object):
     def parse(self, log_entry):
         self.sha1 = log_entry[:40]
         (parents, tags, author, authdate, subject) = \
-                log_entry[41:].split('\0', 5)
+                log_entry[41:].split(chr(0x01), 5)
 
         if subject:
             self.subject = subject
@@ -138,7 +137,6 @@ class RepoReader(object):
             self._topo_list = []
         log_entry = self._proc.stdout.readline()
         if not log_entry:
-            self._proc.kill()
             del self._proc
             self._cached = True
             self._proc = None
