@@ -11,6 +11,7 @@ checkout_branch = 'checkout_branch'
 cherry_pick = 'cherry_pick'
 clone = 'clone'
 delete = 'delete'
+ignore = 'ignore'
 delete_branch = 'delete_branch'
 diff = 'diff'
 diff_expr_mode = 'diff_expr_mode'
@@ -23,7 +24,6 @@ editor_text = 'editor_text'
 format_patch = 'format_patch'
 grep = 'grep'
 information = 'information'
-inotify = 'inotify'
 log_cmd = 'log_cmd'
 load_commit_message = 'load_commit_message'
 load_commit_template = 'load_commit_template'
@@ -54,6 +54,7 @@ unstage_diffs = 'unstage_diffs'
 unstage_all = 'unstage_all'
 unstage_selected = 'unstage_selected'
 untracked_summary = 'untracked_summary'
+update_status = 'update_status'
 visualize_all = 'visualize_all'
 visualize_current = 'visualize_current'
 visualize_paths = 'visualize_paths'
