@@ -1,11 +1,7 @@
-import os
-import sys
-
 from PyQt4 import QtGui
 from PyQt4 import QtCore
 from PyQt4.QtCore import SIGNAL
 
-import cola
 from cola import qtutils
 from cola.views import standard
 
