@@ -9,20 +9,49 @@ if __name__ == "__main__":
     src = os.path.join(os.path.dirname(__file__), '..', '..')
     sys.path.insert(0, os.path.abspath(src))
 
+from cola import observable
 from cola import qtutils
+from cola import signals
+from cola import gitcmds
+from cola.compat import set
+from cola.decorators import memoize
 from cola.models import commit
 from cola.views import standard
-from cola.compat import set
+from cola.views import syntax
 
 
 def git_dag(log_args=None, parent=None):
     """Return a pre-populated git DAG widget."""
     view = GitDAGWidget(parent)
-    view.thread.start(QtCore.QThread.LowPriority)
+    view.resize_to_desktop()
+    view.raise_()
     view.show()
+    view.thread.start(QtCore.QThread.LowPriority)
     return view
 
 
+class GitCommitView(QtGui.QWidget):
+    def __init__(self, parent=None, nodecom=None):
+        QtGui.QWidget.__init__(self, parent)
+
+        self.diff = QtGui.QTextEdit()
+        self.diff.setLineWrapMode(QtGui.QTextEdit.NoWrap)
+        self.diff.setReadOnly(True)
+        self.diff_syn = syntax.DiffSyntaxHighlighter(self.diff.document())
+        qtutils.set_diff_font(self.diff)
+
+        self._layt = QtGui.QVBoxLayout()
+        self._layt.addWidget(self.diff)
+        self.setLayout(self._layt)
+
+        sig = signals.sha1_selected
+        nodecom.add_message_observer(sig, self._node_selected)
+
+    def _node_selected(self, sha1):
+        self.diff.setText(gitcmds.diff_info(sha1))
+        qtutils.set_clipboard(sha1)
+
+
 class GitDAGWidget(standard.StandardDialog):
     """The git-dag widget."""
     # Keep us in scope otherwise PyQt kills the widget
@@ -31,6 +60,9 @@ class GitDAGWidget(standard.StandardDialog):
     def delete(self):
         self._instances.remove(self)
 
+    def splitter(self):
+        return self._splitter
+
     def __init__(self, parent=None, args=None):
         standard.StandardDialog.__init__(self, parent)
         self._instances.add(self)
@@ -38,12 +70,20 @@ class GitDAGWidget(standard.StandardDialog):
         self.setObjectName('dag')
         self.setWindowTitle(self.tr('git dag'))
         self.setMinimumSize(1, 1)
-        self.resize(777, 666)
 
-        self._graphview = GraphView()
-        layt = QtGui.QHBoxLayout()
-        layt.setMargin(1)
-        layt.addWidget(self._graphview)
+        self._splitter = QtGui.QSplitter()
+        self._splitter.setOrientation(QtCore.Qt.Vertical)
+
+        self._nodecom = observable.Observable()
+        self._graphview = GraphView(nodecom=self._nodecom)
+        self._widget = GitCommitView(nodecom=self._nodecom)
+
+        self._splitter.insertWidget(0, self._graphview)
+        self._splitter.insertWidget(1, self._widget)
+
+        self._layt = layt = QtGui.QHBoxLayout()
+        layt.setMargin(0)
+        layt.addWidget(self._splitter)
         self.setLayout(layt)
 
         qtutils.add_close_action(self)
@@ -51,8 +91,7 @@ class GitDAGWidget(standard.StandardDialog):
             qtutils.center_on_screen(self)
 
         self.thread = ReaderThread(self, args)
-        self.thread.connect(self.thread,
-                            self.thread.commit_ready,
+        self.thread.connect(self.thread, self.thread.commit_ready,
                             self._add_commit)
 
     def _add_commit(self, sha1):
@@ -78,6 +117,13 @@ class GitDAGWidget(standard.StandardDialog):
         self.thread.mutex.unlock()
         self.thread.condition.wakeOne()
 
+    def resize_to_desktop(self):
+        desktop = QtGui.QApplication.instance().desktop()
+        width = desktop.width()
+        height = desktop.height()
+        self.resize(width/2, height)
+        self.splitter().setSizes([height/2, height/2])
+
 
 class ReaderThread(QtCore.QThread):
 
@@ -122,8 +168,8 @@ class Edge(QtGui.QGraphicsItem):
         self.setZValue(-2)
         self.adjust()
 
-    def type(self):
-        return Edge._type
+    def type(self, _type=_type):
+        return _type
 
     def adjust(self):
         if not self.source or not self.dest:
@@ -145,6 +191,7 @@ class Edge(QtGui.QGraphicsItem):
         self.source_pt = line.p1() + offset
         self.dest_pt = line.p2() - offset
 
+    @memoize
     def boundingRect(self, _extra=_arrow_extra):
         if not self.source or not self.dest:
             return QtCore.QRectF()
@@ -190,17 +237,25 @@ class Edge(QtGui.QGraphicsItem):
 
 class Node(QtGui.QGraphicsItem):
     _type = QtGui.QGraphicsItem.UserType + 1
+    _width = 180
+    _height = 18
 
-    def __init__(self, graph, commit):
+    _shape = QtGui.QPainterPath()
+    _shape.addRect(_width/-2., _height/-2., _width, _height)
+
+    _bound = _shape.boundingRect()
+    _glyph = QtCore.QRectF(-_width/2., -9, _width/4., 18)
+
+    def __init__(self, commit, nodecom):
         QtGui.QGraphicsItem.__init__(self)
         self.setZValue(0)
         self.setFlag(QtGui.QGraphicsItem.ItemIsSelectable)
+
         self.commit = commit
-        self._graph = graph
-        self._width = 180
+        self._nodecom = nodecom
+
         # Starts with enough space for two tags. Any more and the node
         # needs to be taller to accomodate.
-        self._height = 18
         if len(self.commit.tags) > 1:
             self._height = len(self.commit.tags) * 9 + 6 # +6 padding
         self._edges = []
@@ -209,7 +264,10 @@ class Node(QtGui.QGraphicsItem):
         self._colors['bg'] = QtGui.QColor.fromRgb(16, 16, 16)
         self._colors['selected'] = QtGui.QColor.fromRgb(192, 192, 16)
         self._colors['outline'] = QtGui.QColor.fromRgb(0, 0, 0)
-        self._colors['node'] = QtGui.QColor.fromRgb(255, 111, 69)
+        if len(commit.parents) == 1:
+            self._colors['node'] = QtGui.QColor.fromRgb(255, 111, 69)
+        else:
+            self._colors['node'] = QtGui.QColor.fromRgb(169, 111, 69)
         self._colors['decorations'] = QtGui.QColor.fromRgb(255, 255, 42)
 
         self._grad = QtGui.QLinearGradient(0.0, 0.0, 0.0, self._height)
@@ -220,23 +278,19 @@ class Node(QtGui.QGraphicsItem):
         self.dragged = False
         self.skipped = False
 
-    def type(self):
-        return Node._type
+    def type(self, _type=_type):
+        return _type
 
     def add_edge(self, edge):
         self._edges.append(edge)
-        edge.adjust()
 
-    def boundingRect(self):
-        return self.shape().boundingRect()
+    def boundingRect(self, _bound=_bound):
+        return _bound
 
-    def shape(self):
-        path = QtGui.QPainterPath()
-        path.addRect(-self._width/2., -self._height/2.,
-                     self._width, self._height)
-        return path
+    def shape(self, _shape=_shape):
+        return _shape
 
-    def glyph(self):
+    def glyph(self, _glyph=_glyph):
         """Provides location of the glyph representing this node
 
         The node contains a glyph (a circle or ellipse) representing the
@@ -245,26 +299,23 @@ class Node(QtGui.QGraphicsItem):
         make edges point at the center of the glyph, rather than at the
         center of the entire node.
         """
-        glyph = QtCore.QRectF(-self._width/2., -9,
-                              self._width/4., 18)
-        return glyph
+        return _glyph
 
     def paint(self, painter, option, widget):
         if self.isSelected():
-            self.setZValue(1)
             painter.setPen(self._colors['selected'])
         else:
-            self.setZValue(0)
             painter.setPen(self._colors['outline'])
         painter.setBrush(self._grad)
 
         # Draw glyph
         painter.drawEllipse(self.glyph())
-        sha1_text = self.commit.sha1
+        sha1_text = self.commit.sha1[:8]
         font = painter.font()
         font.setPointSize(5)
         painter.setFont(font)
         painter.setPen(QtCore.Qt.black)
+
         text_options = QtGui.QTextOption()
         text_options.setAlignment(QtCore.Qt.AlignCenter)
         painter.drawText(self.glyph(), sha1_text, text_options)
@@ -283,23 +334,21 @@ class Node(QtGui.QGraphicsItem):
         painter.translate(2.,0.)
         painter.drawText(text_box, tag_text, text_options)
 
-
-    def itemChange(self, change, value):
-        if change == QtGui.QGraphicsItem.ItemPositionChange:
-            for edge in self._edges:
-                edge.adjust()
-            #self._graph.itemMoved()
-        return QtGui.QGraphicsItem.itemChange(self, change, value)
-
     def mousePressEvent(self, event):
-        self.selected = self.isSelected()
         self.pressed = True
+        self.selected = self.isSelected()
+        sig = signals.sha1_selected
         QtGui.QGraphicsItem.mousePressEvent(self, event)
+        self._nodecom.notify_message_observers(sig, self.commit.sha1)
 
     def mouseMoveEvent(self, event):
         if self.pressed:
             self.dragged = True
         QtGui.QGraphicsItem.mouseMoveEvent(self, event)
+        for node in self.scene().selectedItems():
+            for edge in node._edges:
+                edge.adjust()
+        self.scene().update()
 
     def mouseReleaseEvent(self, event):
         QtGui.QGraphicsItem.mouseReleaseEvent(self, event)
@@ -315,7 +364,7 @@ class Node(QtGui.QGraphicsItem):
 
 
 class GraphView(QtGui.QGraphicsView):
-    def __init__(self):
+    def __init__(self, nodecom):
         QtGui.QGraphicsView.__init__(self)
 
         self._xoff = 200
@@ -325,9 +374,8 @@ class GraphView(QtGui.QGraphicsView):
 
         self._items = []
         self._selected = []
-        self._commits = {}
-        self._children = {}
         self._nodes = {}
+        self._nodecom = nodecom
 
         self._loc = {}
         self._cols = {}
@@ -335,20 +383,18 @@ class GraphView(QtGui.QGraphicsView):
         self._panning = False
         self._last_mouse = [0, 0]
 
-        self._zoom = 1
+        self._zoom = 2
         self.scale(self._zoom, self._zoom)
         self.setDragMode(self.RubberBandDrag)
 
-        size = 30000
         scene = QtGui.QGraphicsScene(self)
         scene.setItemIndexMethod(QtGui.QGraphicsScene.NoIndex)
-        scene.setSceneRect(-size/4, -size/2, size/2, size)
         self.setScene(scene)
 
         self.setCacheMode(QtGui.QGraphicsView.CacheBackground)
         self.setRenderHint(QtGui.QPainter.Antialiasing)
         self.setTransformationAnchor(QtGui.QGraphicsView.AnchorUnderMouse)
-        self.setResizeAnchor(QtGui.QGraphicsView.AnchorViewCenter)
+        self.setResizeAnchor(QtGui.QGraphicsView.NoAnchor)
         self.setBackgroundColor()
 
     def add_commits(self, commits):
@@ -359,6 +405,7 @@ class GraphView(QtGui.QGraphicsView):
 
     def keyPressEvent(self, event):
         key = event.key()
+        QtGui.QGraphicsView.keyPressEvent(self, event)
 
         if key == QtCore.Qt.Key_Plus:
             self._scale_view(1.5)
@@ -366,10 +413,6 @@ class GraphView(QtGui.QGraphicsView):
             self._scale_view(1 / 1.5)
         elif key == QtCore.Qt.Key_F:
             self._view_fit()
-        elif event.key() == QtCore.Qt.Key_Z:
-            self._move_nodes_to_mouse_position()
-        else:
-            QtGui.QGraphicsView.keyPressEvent(self, event)
 
     def _view_fit(self):
         """Fit selected items into the viewport"""
@@ -484,7 +527,7 @@ class GraphView(QtGui.QGraphicsView):
                         .scale(zoom, zoom)
                         .mapRect(QtCore.QRectF(0.0, 0.0, 1.0, 1.0))
                         .width())
-        if factor < 0.02 or factor > 42.0:
+        if factor < 0.014 or factor > 42.0:
             return
         self.setTransformationAnchor(QtGui.QGraphicsView.AnchorUnderMouse)
         self._zoom = zoom
@@ -494,10 +537,9 @@ class GraphView(QtGui.QGraphicsView):
         """Handle mouse wheel panning."""
 
         if event.delta() < 0:
-            s = -100.0
+            s = -133.
         else:
-            s = 100.0
-
+            s = 133.
         pan_rect = QtCore.QRectF(0.0, 0.0, 1.0, 1.0)
         factor = 1.0 / self.matrix().mapRect(pan_rect).width()
 
@@ -505,33 +547,9 @@ class GraphView(QtGui.QGraphicsView):
             matrix = self.matrix().translate(0, s * factor)
         else:
             matrix = self.matrix().translate(s * factor, 0)
-
         self.setTransformationAnchor(QtGui.QGraphicsView.NoAnchor)
         self.setMatrix(matrix)
 
-    def _move_nodes_to_mouse_position(self):
-        items = self.scene().selectedItems()
-        if not items:
-            return
-        dx = 0
-        dy = 0
-        min_distance = sys.maxint
-        for item in items:
-            width = item.boundingRect().width()
-            pos = item.pos()
-            tmp_dx = self._last_mouse[0] - pos.x() - width/2.0
-            tmp_dy = self._last_mouse[1] - pos.y() - width/2.0
-            distance = math.sqrt(tmp_dx ** 2 + tmp_dy ** 2)
-            if distance < min_distance:
-                min_distance = distance
-                dx = tmp_dx
-                dy = tmp_dy
-        for item in items:
-            pos = item.pos()
-            x = pos.x();
-            y = pos.y()
-            item.setPos( x + dx, y + dy )
-
     def setBackgroundColor(self, color=None):
         # To set a gradient background brush we need to use StretchToDeviceMode
         # but that seems to be segfaulting. Use a solid background.
@@ -551,11 +569,7 @@ class GraphView(QtGui.QGraphicsView):
     def add(self, commits):
         scene = self.scene()
         for commit in commits:
-            self._commits[commit.sha1] = commit
-            for p in commit.parents:
-                children = self._children.setdefault(p, [])
-                children.append(commit.sha1)
-            node = Node(self, commit)
+            node = Node(commit, self._nodecom)
             scene.addItem(node)
             self._nodes[commit.sha1] = node
             self._items.append(node)
@@ -564,13 +578,9 @@ class GraphView(QtGui.QGraphicsView):
         """Create edges linking commits with their parents"""
         scene = self.scene()
         for commit in commits:
-            children = self._children.get(commit.sha1, None)
-            # root commit
-            if children is None:
-                continue
             commit_node = self._nodes[commit.sha1]
-            for child_sha1 in children:
-                child_node = self._nodes[child_sha1]
+            for child in commit.children:
+                child_node = self._nodes[child.sha1]
                 edge = Edge(commit_node, child_node)
                 scene.addItem(edge)
 
@@ -578,20 +588,12 @@ class GraphView(QtGui.QGraphicsView):
         gxmax = self._xmax
         gymax = self._ymax
 
-        xpos = 0
-        ypos = 0
         for commit in commits:
-            if commit.sha1 not in self._children:
-                self._loc[commit.sha1] = (xpos, ypos)
-                node = self._nodes.get(commit.sha1, None)
-                node.setPos(xpos, ypos)
-                xpos += self._xoff
-                gxmax = max(xpos, gxmax)
-                continue
+            # Center nodes relative to their children
             ymax = 0
             xmax = None
-            for sha1 in self._children[commit.sha1]:
-                loc = self._loc[sha1]
+            for child in commit.children:
+                loc = self._loc[child.sha1]
                 if xmax is None:
                     xmax = loc[0]
                 xmax = min(xmax, loc[0])
@@ -607,7 +609,11 @@ class GraphView(QtGui.QGraphicsView):
                 gxmax = max(gxmax, xmax)
                 self._cols[ymax] = xmax
             else:
-                xmax = max(0, xmax)
+                prev = ymax - self._yoff
+                if prev in self._cols:
+                    xmax = self._cols[prev]/2
+                else:
+                    xmax = 0
                 self._cols[ymax] = xmax
 
             sha1 = commit.sha1
@@ -615,13 +621,17 @@ class GraphView(QtGui.QGraphicsView):
             node = self._nodes[sha1]
             node.setPos(xmax, ymax)
 
-        xpad = 200
-        ypad = 66
+        xpad = 99
         self._xmax = gxmax
         self._ymax = gymax
-        self.scene().setSceneRect(-xpad, -ypad, gxmax+xpad, gymax+ypad)
+        self.scene().setSceneRect(-xpad, 0, gxmax, gymax)
+
 
 if __name__ == "__main__":
+    from cola.models import main
+
+    model = main.model()
+    model._init_config_data()
     app = QtGui.QApplication(sys.argv)
     view = git_dag()
     sys.exit(app.exec_())
